@@ -3,8 +3,8 @@ import warnings
 
 import numpy as np
 
-from .. import DataArray
 from ..core import indexing
+from ..core.dataarray import DataArray
 from ..core.utils import is_scalar
 from .common import BackendArray
 from .file_manager import CachingFileManager
@@ -50,11 +50,11 @@ class RasterioArrayWrapper(BackendArray):
         return self._shape
 
     def _get_indexer(self, key):
-        """ Get indexer for rasterio array.
+        """Get indexer for rasterio array.
 
-        Parameter
-        ---------
-        key: tuple of int
+        Parameters
+        ----------
+        key : tuple of int
 
         Returns
         -------
@@ -63,7 +63,7 @@ class RasterioArrayWrapper(BackendArray):
         squeeze_axis: axes to be squeezed
         np_ind: indexer for loaded numpy array
 
-        See also
+        See Also
         --------
         indexing.decompose_indexer
         """
@@ -174,12 +174,46 @@ def open_rasterio(filename, parse_coordinates=None, chunks=None, cache=None, loc
 
     You can generate 2D coordinates from the file's attributes with::
 
-        from affine import Affine
-        da = xr.open_rasterio('path_to_file.tif')
-        transform = Affine.from_gdal(*da.attrs['transform'])
-        nx, ny = da.sizes['x'], da.sizes['y']
-        x, y = np.meshgrid(np.arange(nx)+0.5, np.arange(ny)+0.5) * transform
-
+        >>> from affine import Affine
+        >>> da = xr.open_rasterio(
+        ...     "https://github.com/mapbox/rasterio/raw/1.2.1/tests/data/RGB.byte.tif"
+        ... )
+        >>> da
+        <xarray.DataArray (band: 3, y: 718, x: 791)>
+        [1703814 values with dtype=uint8]
+        Coordinates:
+          * band     (band) int64 1 2 3
+          * y        (y) float64 2.827e+06 2.826e+06 2.826e+06 ... 2.612e+06 2.612e+06
+          * x        (x) float64 1.021e+05 1.024e+05 1.027e+05 ... 3.389e+05 3.392e+05
+        Attributes:
+            transform:      (300.0379266750948, 0.0, 101985.0, 0.0, -300.041782729805...
+            crs:            +init=epsg:32618
+            res:            (300.0379266750948, 300.041782729805)
+            is_tiled:       0
+            nodatavals:     (0.0, 0.0, 0.0)
+            scales:         (1.0, 1.0, 1.0)
+            offsets:        (0.0, 0.0, 0.0)
+            AREA_OR_POINT:  Area
+        >>> transform = Affine(*da.attrs["transform"])
+        >>> transform
+        Affine(300.0379266750948, 0.0, 101985.0,
+               0.0, -300.041782729805, 2826915.0)
+        >>> nx, ny = da.sizes["x"], da.sizes["y"]
+        >>> x, y = transform * np.meshgrid(np.arange(nx) + 0.5, np.arange(ny) + 0.5)
+        >>> x
+        array([[102135.01896334, 102435.05689001, 102735.09481669, ...,
+                338564.90518331, 338864.94310999, 339164.98103666],
+               [102135.01896334, 102435.05689001, 102735.09481669, ...,
+                338564.90518331, 338864.94310999, 339164.98103666],
+               [102135.01896334, 102435.05689001, 102735.09481669, ...,
+                338564.90518331, 338864.94310999, 339164.98103666],
+               ...,
+               [102135.01896334, 102435.05689001, 102735.09481669, ...,
+                338564.90518331, 338864.94310999, 339164.98103666],
+               [102135.01896334, 102435.05689001, 102735.09481669, ...,
+                338564.90518331, 338864.94310999, 339164.98103666],
+               [102135.01896334, 102435.05689001, 102735.09481669, ...,
+                338564.90518331, 338864.94310999, 339164.98103666]])
 
     Parameters
     ----------
@@ -221,14 +255,17 @@ def open_rasterio(filename, parse_coordinates=None, chunks=None, cache=None, loc
         vrt = filename
         filename = vrt.src_dataset.name
         vrt_params = dict(
+            src_crs=vrt.src_crs.to_string(),
             crs=vrt.crs.to_string(),
             resampling=vrt.resampling,
-            src_nodata=vrt.src_nodata,
-            dst_nodata=vrt.dst_nodata,
             tolerance=vrt.tolerance,
-            transform=vrt.transform,
+            src_nodata=vrt.src_nodata,
+            nodata=vrt.nodata,
             width=vrt.width,
             height=vrt.height,
+            src_transform=vrt.src_transform,
+            transform=vrt.transform,
+            dtype=vrt.working_dtype,
             warp_extras=vrt.warp_extras,
         )
 
@@ -333,9 +370,7 @@ def open_rasterio(filename, parse_coordinates=None, chunks=None, cache=None, loc
             else:
                 attrs[k] = v
 
-    data = indexing.LazilyOuterIndexedArray(
-        RasterioArrayWrapper(manager, lock, vrt_params)
-    )
+    data = indexing.LazilyIndexedArray(RasterioArrayWrapper(manager, lock, vrt_params))
 
     # this lets you write arrays loaded with rasterio
     data = indexing.CopyOnWriteArray(data)
@@ -358,6 +393,6 @@ def open_rasterio(filename, parse_coordinates=None, chunks=None, cache=None, loc
         result = result.chunk(chunks, name_prefix=name_prefix, token=token)
 
     # Make the file closeable
-    result._file_obj = manager
+    result.set_close(manager.close)
 
     return result
