@@ -1,14 +1,29 @@
-from io import BytesIO
+import io
+import os
 
 import numpy as np
 
-from .. import Variable
 from ..core.indexing import NumpyIndexingAdapter
-from ..core.utils import Frozen, FrozenDict
-from .common import BackendArray, WritableCFDataStore
+from ..core.utils import Frozen, FrozenDict, close_on_error, read_magic_number
+from ..core.variable import Variable
+from .common import (
+    BACKEND_ENTRYPOINTS,
+    BackendArray,
+    BackendEntrypoint,
+    WritableCFDataStore,
+    _normalize_path,
+)
 from .file_manager import CachingFileManager, DummyFileManager
 from .locks import ensure_lock, get_write_lock
 from .netcdf3 import encode_nc3_attr_value, encode_nc3_variable, is_valid_nc3_name
+from .store import StoreBackendEntrypoint
+
+try:
+    import scipy.io
+
+    has_scipy = True
+except ModuleNotFoundError:
+    has_scipy = False
 
 
 def _decode_string(s):
@@ -57,7 +72,6 @@ class ScipyArrayWrapper(BackendArray):
 
 
 def _open_scipy_netcdf(filename, mode, mmap, version):
-    import scipy.io
     import gzip
 
     # if the string ends with .gz, then gunzip and open as netcdf file
@@ -69,15 +83,13 @@ def _open_scipy_netcdf(filename, mode, mmap, version):
         except TypeError as e:
             # TODO: gzipped loading only works with NetCDF3 files.
             if "is not a valid NetCDF 3 file" in e.message:
-                raise ValueError(
-                    "gzipped file loading only supports " "NetCDF 3 files."
-                )
+                raise ValueError("gzipped file loading only supports NetCDF 3 files.")
             else:
                 raise
 
     if isinstance(filename, bytes) and filename.startswith(b"CDF"):
         # it's a NetCDF3 bytestring
-        filename = BytesIO(filename)
+        filename = io.BytesIO(filename)
 
     try:
         return scipy.io.netcdf_file(filename, mode=mode, mmap=mmap, version=version)
@@ -109,9 +121,7 @@ class ScipyDataStore(WritableCFDataStore):
         self, filename_or_obj, mode="r", format=None, group=None, mmap=None, lock=None
     ):
         if group is not None:
-            raise ValueError(
-                "cannot save to a group with the " "scipy.io.netcdf backend"
-            )
+            raise ValueError("cannot save to a group with the scipy.io.netcdf backend")
 
         if format is None or format == "NETCDF3_64BIT":
             version = 2
@@ -221,3 +231,57 @@ class ScipyDataStore(WritableCFDataStore):
 
     def close(self):
         self._manager.close()
+
+
+class ScipyBackendEntrypoint(BackendEntrypoint):
+    def guess_can_open(self, filename_or_obj):
+        try:
+            return read_magic_number(filename_or_obj).startswith(b"CDF")
+        except TypeError:
+            pass
+
+        try:
+            _, ext = os.path.splitext(filename_or_obj)
+        except TypeError:
+            return False
+        return ext in {".nc", ".nc4", ".cdf", ".gz"}
+
+    def open_dataset(
+        self,
+        filename_or_obj,
+        mask_and_scale=True,
+        decode_times=True,
+        concat_characters=True,
+        decode_coords=True,
+        drop_variables=None,
+        use_cftime=None,
+        decode_timedelta=None,
+        mode="r",
+        format=None,
+        group=None,
+        mmap=None,
+        lock=None,
+    ):
+
+        filename_or_obj = _normalize_path(filename_or_obj)
+        store = ScipyDataStore(
+            filename_or_obj, mode=mode, format=format, group=group, mmap=mmap, lock=lock
+        )
+
+        store_entrypoint = StoreBackendEntrypoint()
+        with close_on_error(store):
+            ds = store_entrypoint.open_dataset(
+                store,
+                mask_and_scale=mask_and_scale,
+                decode_times=decode_times,
+                concat_characters=concat_characters,
+                decode_coords=decode_coords,
+                drop_variables=drop_variables,
+                use_cftime=use_cftime,
+                decode_timedelta=decode_timedelta,
+            )
+        return ds
+
+
+if has_scipy:
+    BACKEND_ENTRYPOINTS["scipy"] = ScipyBackendEntrypoint
