@@ -7,7 +7,7 @@ import pytest
 from xarray import DataArray, Dataset, Variable
 from xarray.core import indexing, nputils
 
-from . import IndexerMaker, ReturnItem, assert_array_equal, raises_regex
+from . import IndexerMaker, ReturnItem, assert_array_equal
 
 B = IndexerMaker(indexing.BasicIndexer)
 
@@ -21,24 +21,24 @@ class TestIndexers:
     def test_expanded_indexer(self):
         x = np.random.randn(10, 11, 12, 13, 14)
         y = np.arange(5)
-        I = ReturnItem()  # noqa
+        arr = ReturnItem()
         for i in [
-            I[:],
-            I[...],
-            I[0, :, 10],
-            I[..., 10],
-            I[:5, ..., 0],
-            I[..., 0, :],
-            I[y],
-            I[y, y],
-            I[..., y, y],
-            I[..., 0, 1, 2, 3, 4],
+            arr[:],
+            arr[...],
+            arr[0, :, 10],
+            arr[..., 10],
+            arr[:5, ..., 0],
+            arr[..., 0, :],
+            arr[y],
+            arr[y, y],
+            arr[..., y, y],
+            arr[..., 0, 1, 2, 3, 4],
         ]:
             j = indexing.expanded_indexer(i, x.ndim)
             assert_array_equal(x[i], x[j])
             assert_array_equal(self.set_to_zero(x, i), self.set_to_zero(x, j))
-        with raises_regex(IndexError, "too many indices"):
-            indexing.expanded_indexer(I[1, 2, 3], 2)
+        with pytest.raises(IndexError, match=r"too many indices"):
+            indexing.expanded_indexer(arr[1, 2, 3], 2)
 
     def test_asarray_tuplesafe(self):
         res = indexing._asarray_tuplesafe(("a", 1))
@@ -69,24 +69,32 @@ class TestIndexers:
     def test_convert_label_indexer(self):
         # TODO: add tests that aren't just for edge cases
         index = pd.Index([1, 2, 3])
-        with raises_regex(KeyError, "not all values found"):
+        with pytest.raises(KeyError, match=r"not all values found"):
             indexing.convert_label_indexer(index, [0])
         with pytest.raises(KeyError):
             indexing.convert_label_indexer(index, 0)
-        with raises_regex(ValueError, "does not have a MultiIndex"):
+        with pytest.raises(ValueError, match=r"does not have a MultiIndex"):
             indexing.convert_label_indexer(index, {"one": 0})
 
         mindex = pd.MultiIndex.from_product([["a", "b"], [1, 2]], names=("one", "two"))
-        with raises_regex(KeyError, "not all values found"):
+        with pytest.raises(KeyError, match=r"not all values found"):
             indexing.convert_label_indexer(mindex, [0])
         with pytest.raises(KeyError):
             indexing.convert_label_indexer(mindex, 0)
         with pytest.raises(ValueError):
             indexing.convert_label_indexer(index, {"three": 0})
-        with pytest.raises((KeyError, IndexError)):
-            # pandas 0.21 changed this from KeyError to IndexError
+        with pytest.raises(IndexError):
             indexing.convert_label_indexer(mindex, (slice(None), 1, "no_level"))
 
+    def test_convert_label_indexer_datetime(self):
+        index = pd.to_datetime(["2000-01-01", "2001-01-01", "2002-01-01"])
+        actual = indexing.convert_label_indexer(index, "2001-01-01")
+        expected = (1, None)
+        assert actual == expected
+
+        actual = indexing.convert_label_indexer(index, index.to_numpy()[1])
+        assert actual == expected
+
     def test_convert_unsorted_datetime_index_raises(self):
         index = pd.to_datetime(["2001", "2000", "2002"])
         with pytest.raises(KeyError):
@@ -102,13 +110,13 @@ class TestIndexers:
         dim_indexers = indexing.get_dim_indexers(mdata, {"one": "a", "two": 1})
         assert dim_indexers == {"x": {"one": "a", "two": 1}}
 
-        with raises_regex(ValueError, "cannot combine"):
+        with pytest.raises(ValueError, match=r"cannot combine"):
             indexing.get_dim_indexers(mdata, {"x": "a", "two": 1})
 
-        with raises_regex(ValueError, "do not exist"):
+        with pytest.raises(ValueError, match=r"do not exist"):
             indexing.get_dim_indexers(mdata, {"y": "a"})
 
-        with raises_regex(ValueError, "do not exist"):
+        with pytest.raises(ValueError, match=r"do not exist"):
             indexing.get_dim_indexers(mdata, {"four": 1})
 
     def test_remap_label_indexers(self):
@@ -169,14 +177,13 @@ class TestIndexers:
         )
 
     def test_read_only_view(self):
-        from collections import OrderedDict
 
         arr = DataArray(
             np.random.rand(3, 3),
             coords={"x": np.arange(3), "y": np.arange(3)},
             dims=("x", "y"),
         )  # Create a 2D DataArray
-        arr = arr.expand_dims(OrderedDict([("z", 3)]), -1)  # New dimension 'z'
+        arr = arr.expand_dims({"z": 3}, -1)  # New dimension 'z'
         arr["z"] = np.arange(3)  # New coords to dimension 'z'
         with pytest.raises(ValueError, match="Do you want to .copy()"):
             arr.loc[0, 0, 0] = 999
@@ -184,27 +191,27 @@ class TestIndexers:
 
 class TestLazyArray:
     def test_slice_slice(self):
-        I = ReturnItem()  # noqa: E741  # allow ambiguous name
+        arr = ReturnItem()
         for size in [100, 99]:
             # We test even/odd size cases
             x = np.arange(size)
             slices = [
-                I[:3],
-                I[:4],
-                I[2:4],
-                I[:1],
-                I[:-1],
-                I[5:-1],
-                I[-5:-1],
-                I[::-1],
-                I[5::-1],
-                I[:3:-1],
-                I[:30:-1],
-                I[10:4:],
-                I[::4],
-                I[4:4:4],
-                I[:4:-4],
-                I[::-2],
+                arr[:3],
+                arr[:4],
+                arr[2:4],
+                arr[:1],
+                arr[:-1],
+                arr[5:-1],
+                arr[-5:-1],
+                arr[::-1],
+                arr[5::-1],
+                arr[:3:-1],
+                arr[:30:-1],
+                arr[10:4:],
+                arr[::4],
+                arr[4:4:4],
+                arr[:4:-4],
+                arr[::-2],
             ]
             for i in slices:
                 for j in slices:
@@ -217,11 +224,11 @@ class TestLazyArray:
         original = np.random.rand(10, 20, 30)
         x = indexing.NumpyIndexingAdapter(original)
         v = Variable(["i", "j", "k"], original)
-        lazy = indexing.LazilyOuterIndexedArray(x)
+        lazy = indexing.LazilyIndexedArray(x)
         v_lazy = Variable(["i", "j", "k"], lazy)
-        I = ReturnItem()  # noqa: E741  # allow ambiguous name
+        arr = ReturnItem()
         # test orthogonally applied indexers
-        indexers = [I[:], 0, -2, I[:3], [0, 1, 2, 3], [0], np.arange(10) < 5]
+        indexers = [arr[:], 0, -2, arr[:3], [0, 1, 2, 3], [0], np.arange(10) < 5]
         for i in indexers:
             for j in indexers:
                 for k in indexers:
@@ -237,9 +244,7 @@ class TestLazyArray:
                     ]:
                         assert expected.shape == actual.shape
                         assert_array_equal(expected, actual)
-                        assert isinstance(
-                            actual._data, indexing.LazilyOuterIndexedArray
-                        )
+                        assert isinstance(actual._data, indexing.LazilyIndexedArray)
 
                         # make sure actual.key is appropriate type
                         if all(
@@ -252,12 +257,12 @@ class TestLazyArray:
         # test sequentially applied indexers
         indexers = [
             (3, 2),
-            (I[:], 0),
-            (I[:2], -1),
-            (I[:4], [0]),
+            (arr[:], 0),
+            (arr[:2], -1),
+            (arr[:4], [0]),
             ([4, 5], 0),
             ([0, 1, 2], [0, 1]),
-            ([0, 3, 5], I[:2]),
+            ([0, 3, 5], arr[:2]),
         ]
         for i, j in indexers:
             expected = v[i][j]
@@ -275,20 +280,20 @@ class TestLazyArray:
                     actual._data,
                     (
                         indexing.LazilyVectorizedIndexedArray,
-                        indexing.LazilyOuterIndexedArray,
+                        indexing.LazilyIndexedArray,
                     ),
                 )
 
-            assert isinstance(actual._data, indexing.LazilyOuterIndexedArray)
+            assert isinstance(actual._data, indexing.LazilyIndexedArray)
             assert isinstance(actual._data.array, indexing.NumpyIndexingAdapter)
 
     def test_vectorized_lazily_indexed_array(self):
         original = np.random.rand(10, 20, 30)
         x = indexing.NumpyIndexingAdapter(original)
         v_eager = Variable(["i", "j", "k"], x)
-        lazy = indexing.LazilyOuterIndexedArray(x)
+        lazy = indexing.LazilyIndexedArray(x)
         v_lazy = Variable(["i", "j", "k"], lazy)
-        I = ReturnItem()  # noqa: E741  # allow ambiguous name
+        arr = ReturnItem()
 
         def check_indexing(v_eager, v_lazy, indexers):
             for indexer in indexers:
@@ -299,7 +304,7 @@ class TestLazyArray:
                     actual._data,
                     (
                         indexing.LazilyVectorizedIndexedArray,
-                        indexing.LazilyOuterIndexedArray,
+                        indexing.LazilyIndexedArray,
                     ),
                 )
                 assert_array_equal(expected, actual)
@@ -307,7 +312,7 @@ class TestLazyArray:
                 v_lazy = actual
 
         # test orthogonal indexing
-        indexers = [(I[:], 0, 1), (Variable("i", [0, 1]),)]
+        indexers = [(arr[:], 0, 1), (Variable("i", [0, 1]),)]
         check_indexing(v_eager, v_lazy, indexers)
 
         # vectorized indexing
@@ -357,19 +362,19 @@ class TestCopyOnWriteArray:
 
 class TestMemoryCachedArray:
     def test_wrapper(self):
-        original = indexing.LazilyOuterIndexedArray(np.arange(10))
+        original = indexing.LazilyIndexedArray(np.arange(10))
         wrapped = indexing.MemoryCachedArray(original)
         assert_array_equal(wrapped, np.arange(10))
         assert isinstance(wrapped.array, indexing.NumpyIndexingAdapter)
 
     def test_sub_array(self):
-        original = indexing.LazilyOuterIndexedArray(np.arange(10))
+        original = indexing.LazilyIndexedArray(np.arange(10))
         wrapped = indexing.MemoryCachedArray(original)
         child = wrapped[B[:5]]
         assert isinstance(child, indexing.MemoryCachedArray)
         assert_array_equal(child, np.arange(5))
         assert isinstance(child.array, indexing.NumpyIndexingAdapter)
-        assert isinstance(wrapped.array, indexing.LazilyOuterIndexedArray)
+        assert isinstance(wrapped.array, indexing.LazilyIndexedArray)
 
     def test_setitem(self):
         original = np.arange(10)
@@ -464,7 +469,7 @@ def test_vectorized_indexer():
     check_slice(indexing.VectorizedIndexer)
     check_array1d(indexing.VectorizedIndexer)
     check_array2d(indexing.VectorizedIndexer)
-    with raises_regex(ValueError, "numbers of dimensions"):
+    with pytest.raises(ValueError, match=r"numbers of dimensions"):
         indexing.VectorizedIndexer(
             (np.array(1, dtype=np.int64), np.arange(5, dtype=np.int64))
         )
@@ -637,13 +642,13 @@ def test_outer_indexer_consistency_with_broadcast_indexes_vectorized():
 
     original = np.random.rand(10, 20, 30)
     v = Variable(["i", "j", "k"], original)
-    I = ReturnItem()  # noqa: E741  # allow ambiguous name
+    arr = ReturnItem()
     # test orthogonally applied indexers
     indexers = [
-        I[:],
+        arr[:],
         0,
         -2,
-        I[:3],
+        arr[:3],
         np.array([0, 1, 2, 3]),
         np.array([0]),
         np.arange(10) < 5,
@@ -729,7 +734,7 @@ def test_create_mask_dask():
 
 
 def test_create_mask_error():
-    with raises_regex(TypeError, "unexpected key type"):
+    with pytest.raises(TypeError, match=r"unexpected key type"):
         indexing.create_mask((1, 2), (3, 4))
 
 
@@ -748,3 +753,16 @@ def test_create_mask_error():
 def test_posify_mask_subindexer(indices, expected):
     actual = indexing._posify_mask_subindexer(indices)
     np.testing.assert_array_equal(expected, actual)
+
+
+def test_indexing_1d_object_array():
+    items = (np.arange(3), np.arange(6))
+    arr = DataArray(np.array(items))
+
+    actual = arr[0]
+
+    expected_data = np.empty((), dtype=object)
+    expected_data[()] = items[0]
+    expected = DataArray(expected_data)
+
+    assert [actual.data.item()] == [expected.data.item()]
