@@ -1,9 +1,18 @@
+from itertools import combinations, permutations
+
 import numpy as np
 import pandas as pd
 import pytest
 
 import xarray as xr
-from xarray.tests import assert_allclose, assert_equal, requires_cftime, requires_scipy
+from xarray.tests import (
+    assert_allclose,
+    assert_equal,
+    assert_identical,
+    requires_cftime,
+    requires_dask,
+    requires_scipy,
+)
 
 from ..coding.cftimeindex import _parse_array_of_cftime_strings
 from . import has_dask, has_scipy
@@ -63,12 +72,6 @@ def test_interpolate_1d(method, dim, case):
 
     da = get_example_data(case)
     xdest = np.linspace(0.0, 0.9, 80)
-
-    if dim == "y" and case == 1:
-        with pytest.raises(NotImplementedError):
-            actual = da.interp(method=method, **{dim: xdest})
-        pytest.skip("interpolation along chunked dimension is " "not yet supported")
-
     actual = da.interp(method=method, **{dim: xdest})
 
     # scipy interpolation for the reference
@@ -187,7 +190,7 @@ def test_interpolate_vectorize(use_dask):
             "w": xdest["w"],
             "z2": xdest["z2"],
             "y": da["y"],
-            "x": (("z", "w"), xdest),
+            "x": (("z", "w"), xdest.data),
             "x2": (("z", "w"), func(da["x2"], "x", xdest)),
         },
     )
@@ -274,6 +277,32 @@ def test_interpolate_nd_nd():
         da.interp(a=ia)
 
 
+@requires_scipy
+def test_interpolate_nd_with_nan():
+    """Interpolate an array with an nd indexer and `NaN` values."""
+
+    # Create indexer into `a` with dimensions (y, x)
+    x = [0, 1, 2]
+    y = [10, 20]
+    c = {"x": x, "y": y}
+    a = np.arange(6, dtype=float).reshape(2, 3)
+    a[0, 1] = np.nan
+    ia = xr.DataArray(a, dims=("y", "x"), coords=c)
+
+    da = xr.DataArray([1, 2, 2], dims=("a"), coords={"a": [0, 2, 4]})
+    out = da.interp(a=ia)
+    expected = xr.DataArray(
+        [[1.0, np.nan, 2.0], [2.0, 2.0, np.nan]], dims=("y", "x"), coords=c
+    )
+    xr.testing.assert_allclose(out.drop_vars("a"), expected)
+
+    db = 2 * da
+    ds = xr.Dataset({"da": da, "db": db})
+    out = ds.interp(a=ia)
+    expected_ds = xr.Dataset({"da": expected, "db": 2 * expected})
+    xr.testing.assert_allclose(out.drop_vars("a"), expected_ds)
+
+
 @pytest.mark.parametrize("method", ["linear"])
 @pytest.mark.parametrize("case", [0, 1])
 def test_interpolate_scalar(method, case):
@@ -376,8 +405,6 @@ def test_errors(use_dask):
     # invalid method
     with pytest.raises(ValueError):
         da.interp(x=[2, 0], method="boo")
-    with pytest.raises(ValueError):
-        da.interp(x=[2, 0], y=2, method="cubic")
     with pytest.raises(ValueError):
         da.interp(y=[2, 0], method="boo")
 
@@ -468,7 +495,7 @@ def test_dataset():
 
 @pytest.mark.parametrize("case", [0, 3])
 def test_interpolate_dimorder(case):
-    """ Make sure the resultant dimension order is consistent with .sel() """
+    """Make sure the resultant dimension order is consistent with .sel()"""
     if not has_scipy:
         pytest.skip("scipy is not installed.")
 
@@ -552,6 +579,7 @@ def test_interp_like():
             [0.5, 1.5],
         ),
         (["2000-01-01T12:00", "2000-01-02T12:00"], [0.5, 1.5]),
+        (["2000-01-01T12:00", "2000-01-02T12:00", "NaT"], [0.5, 1.5, np.nan]),
         (["2000-01-01T12:00"], 0.5),
         pytest.param("2000-01-01T12:00", 0.5, marks=pytest.mark.xfail),
     ],
@@ -714,6 +742,162 @@ def test_decompose(method):
     x_broadcast, y_broadcast = xr.broadcast(x_new, y_new)
     assert x_broadcast.ndim == 2
 
-    actual = da.interp(x=x_new, y=y_new, method=method).drop(("x", "y"))
-    expected = da.interp(x=x_broadcast, y=y_broadcast, method=method).drop(("x", "y"))
+    actual = da.interp(x=x_new, y=y_new, method=method).drop_vars(("x", "y"))
+    expected = da.interp(x=x_broadcast, y=y_broadcast, method=method).drop_vars(
+        ("x", "y")
+    )
     assert_allclose(actual, expected)
+
+
+@requires_scipy
+@requires_dask
+@pytest.mark.parametrize(
+    "method", ["linear", "nearest", "zero", "slinear", "quadratic", "cubic"]
+)
+@pytest.mark.parametrize("chunked", [True, False])
+@pytest.mark.parametrize(
+    "data_ndim,interp_ndim,nscalar",
+    [
+        (data_ndim, interp_ndim, nscalar)
+        for data_ndim in range(1, 4)
+        for interp_ndim in range(1, data_ndim + 1)
+        for nscalar in range(0, interp_ndim + 1)
+    ],
+)
+def test_interpolate_chunk_1d(method, data_ndim, interp_ndim, nscalar, chunked):
+    """Interpolate nd array with multiple independant indexers
+
+    It should do a series of 1d interpolation
+    """
+
+    # 3d non chunked data
+    x = np.linspace(0, 1, 5)
+    y = np.linspace(2, 4, 7)
+    z = np.linspace(-0.5, 0.5, 11)
+    da = xr.DataArray(
+        data=np.sin(x[:, np.newaxis, np.newaxis])
+        * np.cos(y[:, np.newaxis])
+        * np.exp(z),
+        coords=[("x", x), ("y", y), ("z", z)],
+    )
+    kwargs = {"fill_value": "extrapolate"}
+
+    # choose the data dimensions
+    for data_dims in permutations(da.dims, data_ndim):
+
+        # select only data_ndim dim
+        da = da.isel(  # take the middle line
+            {dim: len(da.coords[dim]) // 2 for dim in da.dims if dim not in data_dims}
+        )
+
+        # chunk data
+        da = da.chunk(chunks={dim: i + 1 for i, dim in enumerate(da.dims)})
+
+        # choose the interpolation dimensions
+        for interp_dims in permutations(da.dims, interp_ndim):
+            # choose the scalar interpolation dimensions
+            for scalar_dims in combinations(interp_dims, nscalar):
+                dest = {}
+                for dim in interp_dims:
+                    if dim in scalar_dims:
+                        # take the middle point
+                        dest[dim] = 0.5 * (da.coords[dim][0] + da.coords[dim][-1])
+                    else:
+                        # pick some points, including outside the domain
+                        before = 2 * da.coords[dim][0] - da.coords[dim][1]
+                        after = 2 * da.coords[dim][-1] - da.coords[dim][-2]
+
+                        dest[dim] = np.linspace(before, after, len(da.coords[dim]) * 13)
+                        if chunked:
+                            dest[dim] = xr.DataArray(data=dest[dim], dims=[dim])
+                            dest[dim] = dest[dim].chunk(2)
+                actual = da.interp(method=method, **dest, kwargs=kwargs)
+                expected = da.compute().interp(method=method, **dest, kwargs=kwargs)
+
+                assert_identical(actual, expected)
+
+                # all the combinations are usually not necessary
+                break
+            break
+        break
+
+
+@requires_scipy
+@requires_dask
+@pytest.mark.parametrize("method", ["linear", "nearest"])
+def test_interpolate_chunk_advanced(method):
+    """Interpolate nd array with an nd indexer sharing coordinates."""
+    # Create original array
+    x = np.linspace(-1, 1, 5)
+    y = np.linspace(-1, 1, 7)
+    z = np.linspace(-1, 1, 11)
+    t = np.linspace(0, 1, 13)
+    q = np.linspace(0, 1, 17)
+    da = xr.DataArray(
+        data=np.sin(x[:, np.newaxis, np.newaxis, np.newaxis, np.newaxis])
+        * np.cos(y[:, np.newaxis, np.newaxis, np.newaxis])
+        * np.exp(z[:, np.newaxis, np.newaxis])
+        * t[:, np.newaxis]
+        + q,
+        dims=("x", "y", "z", "t", "q"),
+        coords={"x": x, "y": y, "z": z, "t": t, "q": q, "label": "dummy_attr"},
+    )
+
+    # Create indexer into `da` with shared coordinate ("full-twist" Möbius strip)
+    theta = np.linspace(0, 2 * np.pi, 5)
+    w = np.linspace(-0.25, 0.25, 7)
+    r = xr.DataArray(
+        data=1 + w[:, np.newaxis] * np.cos(theta),
+        coords=[("w", w), ("theta", theta)],
+    )
+
+    x = r * np.cos(theta)
+    y = r * np.sin(theta)
+    z = xr.DataArray(
+        data=w[:, np.newaxis] * np.sin(theta),
+        coords=[("w", w), ("theta", theta)],
+    )
+
+    kwargs = {"fill_value": None}
+    expected = da.interp(t=0.5, x=x, y=y, z=z, kwargs=kwargs, method=method)
+
+    da = da.chunk(2)
+    x = x.chunk(1)
+    z = z.chunk(3)
+    actual = da.interp(t=0.5, x=x, y=y, z=z, kwargs=kwargs, method=method)
+    assert_identical(actual, expected)
+
+
+@requires_scipy
+def test_interp1d_bounds_error():
+    """Ensure exception on bounds error is raised if requested"""
+    da = xr.DataArray(
+        np.sin(0.3 * np.arange(4)),
+        [("time", np.arange(4))],
+    )
+
+    with pytest.raises(ValueError):
+        da.interp(time=3.5, kwargs=dict(bounds_error=True))
+
+    # default is to fill with nans, so this should pass
+    da.interp(time=3.5)
+
+
+@requires_scipy
+@pytest.mark.parametrize(
+    "x, expect_same_attrs",
+    [
+        (2.5, True),
+        (np.array([2.5, 5]), True),
+        (("x", np.array([0, 0.5, 1, 2]), dict(unit="s")), False),
+    ],
+)
+def test_coord_attrs(x, expect_same_attrs):
+    base_attrs = dict(foo="bar")
+    ds = xr.Dataset(
+        data_vars=dict(a=2 * np.arange(5)),
+        coords={"x": ("x", np.arange(5), base_attrs)},
+    )
+
+    has_same_attrs = ds.interp(x=x).x.attrs == base_attrs
+    assert expect_same_attrs == has_same_attrs
